/*
 *  Copyright (C) 2018-2021 Team Kodi (https://kodi.tv)
 *
 *  SPDX-License-Identifier: GPL-2.0-or-later
 *  See LICENSE.md for more information.
 */

#pragma once

#define DEFAULT_KEYBOARD_FEATURE_BACKSPACE       "backspace"
#define DEFAULT_KEYBOARD_FEATURE_TAB             "tab"
#define DEFAULT_KEYBOARD_FEATURE_CLEAR           "clear"
#define DEFAULT_KEYBOARD_FEATURE_ENTER           "enter"
#define DEFAULT_KEYBOARD_FEATURE_PAUSE           "pause"
#define DEFAULT_KEYBOARD_FEATURE_ESCAPE          "escape"
#define DEFAULT_KEYBOARD_FEATURE_SPACE           "space"
#define DEFAULT_KEYBOARD_FEATURE_EXCLAIM         "exclaim"
#define DEFAULT_KEYBOARD_FEATURE_DOUBLEQUOTE     "doublequote"
#define DEFAULT_KEYBOARD_FEATURE_HASH            "hash"
#define DEFAULT_KEYBOARD_FEATURE_DOLLAR          "dollar"
#define DEFAULT_KEYBOARD_FEATURE_AMPERSAND       "ampersand"
#define DEFAULT_KEYBOARD_FEATURE_QUOTE           "quote"
#define DEFAULT_KEYBOARD_FEATURE_LEFTPAREN       "leftparen"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTPAREN      "rightparen"
#define DEFAULT_KEYBOARD_FEATURE_ASTERISK        "asterisk"
#define DEFAULT_KEYBOARD_FEATURE_PLUS            "plus"
#define DEFAULT_KEYBOARD_FEATURE_COMMA           "comma"
#define DEFAULT_KEYBOARD_FEATURE_MINUS           "minus"
#define DEFAULT_KEYBOARD_FEATURE_PERIOD          "period"
#define DEFAULT_KEYBOARD_FEATURE_SLASH           "slash"
#define DEFAULT_KEYBOARD_FEATURE_0               "0"
#define DEFAULT_KEYBOARD_FEATURE_1               "1"
#define DEFAULT_KEYBOARD_FEATURE_2               "2"
#define DEFAULT_KEYBOARD_FEATURE_3               "3"
#define DEFAULT_KEYBOARD_FEATURE_4               "4"
#define DEFAULT_KEYBOARD_FEATURE_5               "5"
#define DEFAULT_KEYBOARD_FEATURE_6               "6"
#define DEFAULT_KEYBOARD_FEATURE_7               "7"
#define DEFAULT_KEYBOARD_FEATURE_8               "8"
#define DEFAULT_KEYBOARD_FEATURE_9               "9"
#define DEFAULT_KEYBOARD_FEATURE_COLON           "colon"
#define DEFAULT_KEYBOARD_FEATURE_SEMICOLON       "semicolon"
#define DEFAULT_KEYBOARD_FEATURE_LESS            "less"
#define DEFAULT_KEYBOARD_FEATURE_EQUALS          "equals"
#define DEFAULT_KEYBOARD_FEATURE_GREATER         "greater"
#define DEFAULT_KEYBOARD_FEATURE_QUESTION        "question"
#define DEFAULT_KEYBOARD_FEATURE_AT              "at"
#define DEFAULT_KEYBOARD_FEATURE_LEFTBRACKET     "leftbracket"
#define DEFAULT_KEYBOARD_FEATURE_BACKSLASH       "backslash"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTBRACKET    "rightbracket"
#define DEFAULT_KEYBOARD_FEATURE_CARET           "caret"
#define DEFAULT_KEYBOARD_FEATURE_UNDERSCORE      "underscore"
#define DEFAULT_KEYBOARD_FEATURE_GRAVE           "grave"
#define DEFAULT_KEYBOARD_FEATURE_A               "a"
#define DEFAULT_KEYBOARD_FEATURE_B               "b"
#define DEFAULT_KEYBOARD_FEATURE_C               "c"
#define DEFAULT_KEYBOARD_FEATURE_D               "d"
#define DEFAULT_KEYBOARD_FEATURE_E               "e"
#define DEFAULT_KEYBOARD_FEATURE_F               "f"
#define DEFAULT_KEYBOARD_FEATURE_G               "g"
#define DEFAULT_KEYBOARD_FEATURE_H               "h"
#define DEFAULT_KEYBOARD_FEATURE_I               "i"
#define DEFAULT_KEYBOARD_FEATURE_J               "j"
#define DEFAULT_KEYBOARD_FEATURE_K               "k"
#define DEFAULT_KEYBOARD_FEATURE_L               "l"
#define DEFAULT_KEYBOARD_FEATURE_M               "m"
#define DEFAULT_KEYBOARD_FEATURE_N               "n"
#define DEFAULT_KEYBOARD_FEATURE_O               "o"
#define DEFAULT_KEYBOARD_FEATURE_P               "p"
#define DEFAULT_KEYBOARD_FEATURE_Q               "q"
#define DEFAULT_KEYBOARD_FEATURE_R               "r"
#define DEFAULT_KEYBOARD_FEATURE_S               "s"
#define DEFAULT_KEYBOARD_FEATURE_T               "t"
#define DEFAULT_KEYBOARD_FEATURE_U               "u"
#define DEFAULT_KEYBOARD_FEATURE_V               "v"
#define DEFAULT_KEYBOARD_FEATURE_W               "w"
#define DEFAULT_KEYBOARD_FEATURE_X               "x"
#define DEFAULT_KEYBOARD_FEATURE_Y               "y"
#define DEFAULT_KEYBOARD_FEATURE_Z               "z"
#define DEFAULT_KEYBOARD_FEATURE_LEFTBRACE       "leftbrace"
#define DEFAULT_KEYBOARD_FEATURE_BAR             "bar"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTBRACE      "rightbrace"
#define DEFAULT_KEYBOARD_FEATURE_TILDE           "tilde"
#define DEFAULT_KEYBOARD_FEATURE_DELETE          "delete"
#define DEFAULT_KEYBOARD_FEATURE_KP0             "kp0"
#define DEFAULT_KEYBOARD_FEATURE_KP1             "kp1"
#define DEFAULT_KEYBOARD_FEATURE_KP2             "kp2"
#define DEFAULT_KEYBOARD_FEATURE_KP3             "kp3"
#define DEFAULT_KEYBOARD_FEATURE_KP4             "kp4"
#define DEFAULT_KEYBOARD_FEATURE_KP5             "kp5"
#define DEFAULT_KEYBOARD_FEATURE_KP6             "kp6"
#define DEFAULT_KEYBOARD_FEATURE_KP7             "kp7"
#define DEFAULT_KEYBOARD_FEATURE_KP8             "kp8"
#define DEFAULT_KEYBOARD_FEATURE_KP9             "kp9"
#define DEFAULT_KEYBOARD_FEATURE_KPPERIOD        "kpperiod"
#define DEFAULT_KEYBOARD_FEATURE_KPDIVIDE        "kpdivide"
#define DEFAULT_KEYBOARD_FEATURE_KPMULTIPLY      "kpmultiply"
#define DEFAULT_KEYBOARD_FEATURE_KPMINUS         "kpminus"
#define DEFAULT_KEYBOARD_FEATURE_KPPLUS          "kpplus"
#define DEFAULT_KEYBOARD_FEATURE_KPENTER         "kpenter"
#define DEFAULT_KEYBOARD_FEATURE_KPEQUALS        "kpequals"
#define DEFAULT_KEYBOARD_FEATURE_UP              "up"
#define DEFAULT_KEYBOARD_FEATURE_DOWN            "down"
#define DEFAULT_KEYBOARD_FEATURE_RIGHT           "right"
#define DEFAULT_KEYBOARD_FEATURE_LEFT            "left"
#define DEFAULT_KEYBOARD_FEATURE_INSERT          "insert"
#define DEFAULT_KEYBOARD_FEATURE_HOME            "home"
#define DEFAULT_KEYBOARD_FEATURE_END             "end"
#define DEFAULT_KEYBOARD_FEATURE_PAGEUP          "pageup"
#define DEFAULT_KEYBOARD_FEATURE_PAGEDOWN        "pagedown"
#define DEFAULT_KEYBOARD_FEATURE_F1              "f1"
#define DEFAULT_KEYBOARD_FEATURE_F2              "f2"
#define DEFAULT_KEYBOARD_FEATURE_F3              "f3"
#define DEFAULT_KEYBOARD_FEATURE_F4              "f4"
#define DEFAULT_KEYBOARD_FEATURE_F5              "f5"
#define DEFAULT_KEYBOARD_FEATURE_F6              "f6"
#define DEFAULT_KEYBOARD_FEATURE_F7              "f7"
#define DEFAULT_KEYBOARD_FEATURE_F8              "f8"
#define DEFAULT_KEYBOARD_FEATURE_F9              "f9"
#define DEFAULT_KEYBOARD_FEATURE_F10             "f10"
#define DEFAULT_KEYBOARD_FEATURE_F11             "f11"
#define DEFAULT_KEYBOARD_FEATURE_F12             "f12"
#define DEFAULT_KEYBOARD_FEATURE_F13             "f13"
#define DEFAULT_KEYBOARD_FEATURE_F14             "f14"
#define DEFAULT_KEYBOARD_FEATURE_F15             "f15"
#define DEFAULT_KEYBOARD_FEATURE_NUMLOCK         "numlock"
#define DEFAULT_KEYBOARD_FEATURE_CAPSLOCK        "capslock"
#define DEFAULT_KEYBOARD_FEATURE_SCROLLLOCK      "scrolllock"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTSHIFT      "rightshift"
#define DEFAULT_KEYBOARD_FEATURE_LEFTSHIFT       "leftshift"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTCTRL       "rightctrl"
#define DEFAULT_KEYBOARD_FEATURE_LEFTCTRL        "leftctrl"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTALT        "rightalt"
#define DEFAULT_KEYBOARD_FEATURE_LEFTALT         "leftalt"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTMETA       "rightmeta"
#define DEFAULT_KEYBOARD_FEATURE_LEFTMETA        "leftmeta"
#define DEFAULT_KEYBOARD_FEATURE_RIGHTSUPER      "rightsuper"
#define DEFAULT_KEYBOARD_FEATURE_LEFTSUPER       "leftsuper"
#define DEFAULT_KEYBOARD_FEATURE_MODE            "mode"
#define DEFAULT_KEYBOARD_FEATURE_COMPOSE         "compose"
#define DEFAULT_KEYBOARD_FEATURE_HELP            "help"
#define DEFAULT_KEYBOARD_FEATURE_PRINTSCREEN     "printscreen"
#define DEFAULT_KEYBOARD_FEATURE_SYSREQ          "sysreq"
#define DEFAULT_KEYBOARD_FEATURE_BREAK           "break"
#define DEFAULT_KEYBOARD_FEATURE_MENU            "menu"
#define DEFAULT_KEYBOARD_FEATURE_POWER           "power"
#define DEFAULT_KEYBOARD_FEATURE_EURO            "euro"
#define DEFAULT_KEYBOARD_FEATURE_UNDO            "undo"
