/*
 *  Copyright (C) 2017-2021 Team Kodi (https://kodi.tv)
 *
 *  SPDX-License-Identifier: GPL-2.0-or-later
 *  See LICENSE.md for more information.
 */

#pragma once

#define DEFAULT_CONTROLLER_FEATURE_A              "a"
#define DEFAULT_CONTROLLER_FEATURE_B              "b"
#define DEFAULT_CONTROLLER_FEATURE_X              "x"
#define DEFAULT_CONTROLLER_FEATURE_Y              "y"
#define DEFAULT_CONTROLLER_FEATURE_START          "start"
#define DEFAULT_CONTROLLER_FEATURE_BACK           "back"
#define DEFAULT_CONTROLLER_FEATURE_UP             "up"
#define DEFAULT_CONTROLLER_FEATURE_DOWN           "down"
#define DEFAULT_CONTROLLER_FEATURE_RIGHT          "right"
#define DEFAULT_CONTROLLER_FEATURE_LEFT           "left"
#define DEFAULT_CONTROLLER_FEATURE_LEFT_BUMPER    "leftbumber"
#define DEFAULT_CONTROLLER_FEATURE_RIGHT_BUMPER   "rightbumper"
#define DEFAULT_CONTROLLER_FEATURE_LEFT_TRIGGER   "lefttrigger"
#define DEFAULT_CONTROLLER_FEATURE_RIGHT_TRIGGER  "righttrigger"
#define DEFAULT_CONTROLLER_FEATURE_LEFT_THUMB     "leftthumb"
#define DEFAULT_CONTROLLER_FEATURE_RIGHT_THUMB    "rightthumb"
#define DEFAULT_CONTROLLER_FEATURE_LEFT_STICK     "leftstick"
#define DEFAULT_CONTROLLER_FEATURE_RIGHT_STICK    "rightstick"
#define DEFAULT_CONTROLLER_FEATURE_LEFT_MOTOR     "leftmotor"
#define DEFAULT_CONTROLLER_FEATURE_RIGHT_MOTOR    "rightmotor"
// Guide button not used
